/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB9.5
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : ET-MINI 74HC595 + Character LCD
 *           : -> DIN   = Digital-4(PD4) = Digital Output
 *           : -> LATCH = Digital-7(PD7) = Digital Output
 *           : -> CLOCK = Digital-8(PB0) = Digital Output
 *           : LCD 4 Bit Mode Interface Circuit with 74HC595  
 *           : Pin 1  = GND = GND                             
 *           : Pin 2  = VCC = +5V                             
 *           : Pin 3  = VO  = Contrast Adjust Voltage         
 *           : Pin 4  = RS  = Q1 (Pin 1 of 74HC595)           
 *           : Pin 5  = RW  = Q2 (Pin 2 of 74HC595)           
 *           : Pin 6  = EN  = Q3 (Pin 3 of 74HC595)           
 *           : Pin 7  = D0  = GND                             
 *           : Pin 8  = D1  = GND                             
 *           : Pin 9  = D2  = GND                             
 *           : Pin 10 = D3  = GND                             
 *           : Pin 11 = D4  = Q4 (Pin 4 of 74HC595)           
 *           : Pin 12 = D4  = Q5 (Pin 5 of 74HC595)           
 *           : Pin 13 = D6  = Q6 (Pin 6 of 74HC595)            
 *           : Pin 14 = D7  = Q7 (Pin 7 of 74HC595)  
 * Function  : LCD 4 Bit : Control By SPI-74HC595 Interface
 *           : Used Charcter Display on LCD
 */

#include <LCDHC595.h>                       // Used 74HC595 Control lCD Library

// Arduino Pins
#define SIN_PIN   4                         // 74HC595 Serial In Pin
#define STR_PIN   7                         // 74HC595 Strobe pin
#define CLK_PIN   8                         // 74HC595 Clock pin

char MyChar[64]={ 0x04,0x0E,0x15,0x04,0x04,0x04,0x04,0x04,    //Char(ADSII-0):Up
                  0x04,0x04,0x04,0x04,0x04,0x15,0x0E,0x04,    //Char(ASCII-1):Down
                  0x0E,0x11,0x09,0x11,0x11,0x11,0x11,0x11,    //Char(ASCII-2): 
                  0x1A,0x1A,0x0A,0x12,0x12,0x12,0x12,0x1E,    //Char(ASCII-3):
                  0x0E,0x11,0x11,0x1D,0x15,0x11,0x11,0x11,    //Char(ASCII-4):
                  0x07,0x07,0x01,0x01,0x09,0x09,0x05,0x03,    //Char(ASCII-5): 
                  0x0E,0x11,0x01,0x0F,0x0D,0x01,0x01,0x01,    //Char(ASCII-6):
                  0x00,0x00,0x0A,0x1F,0x1F,0x0E,0x04,0x00 };  //Char(ASCII-7):Heart

//Create object to Control LCD.  
LCDHC595 lcd = LCDHC595(SIN_PIN, STR_PIN, CLK_PIN); 

void setup() 
{ 
  lcd.Initial();                              // Initial LCD
  lcd.Backlight(1);                         // ON Backlight LED  
  
  // Create My Character = 8 Font
  for (int i = 0; i<64; i++)                  //8 Character (8 Byte : Char)
  {   
    lcd.Command(0x40+i);                      //Start CG-RAM Address:0x00..0x3F  
    lcd.Print(MyChar[i]);                     //Write CG-RAM Data = 8 Character    
  }
}

void loop()
{ 
  lcd.ClearScreen();                          //Clear Screen Display & Set Cursor = Start Line[1]
  lcd.Print("My Char "); 
  lcd.Print(0x00,BYTE);                       //Write CG-RAM Data
  lcd.Print(0x01,BYTE);
  lcd.Print(0x02,BYTE);
  lcd.Print(0x03,BYTE);
  lcd.Print(0x04,BYTE);
  lcd.Print(0x05,BYTE);
  lcd.Print(0x06,BYTE);
  lcd.Print(0x07,BYTE);  
  
  lcd.SetCursor(0x40);                        //Start Display Line2
  lcd.Print("We are ");
  lcd.Print(0x07,BYTE);
  lcd.Print(" Arduino");

  while(1);                                    //Loop Continue
}  
