/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB6.2
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : ET-MINI DC MOTOR(L293D + DC Motor)
 *           : -> IN1 = Digital-5(PD5) = Digital Output
 *           : -> EN  = Digital-6(PD6) = PWM Output
 *           : -> IN2 = Digital-7(PD7) = Digital Output
 * Function  : Read Analog & Control Speed Motor & Sw Select Direction
 *           : Analog-0   = Volumr  -> Generate Speed Level
 *           : Digital-2  = Switch  -> Switch Select Direction
 *           : Digital-5  = L293D   -> Direction Control
 *           : Digital-7  = L293D   -> Direction Control
 *           : Digital-6  = L293D   -> Control DC Motor Speed 
 *           : Digital-13 = LED Out -> Right Direction Status
 *           : Digital-9  = LED Out -> Left  Direction Status
 */

int IN1 = 5;                                                //L293D  Direction-1
int EN  = 6;                                                //L293D  PWM Enable Control Speed
int IN2 = 7;                                                //L293D  Direction-2
int Volume = 0;                                             //Analog Volume Adjust Speed

int ledLeft = 9;                                            //LED Left Direction Status
int ledRight = 13;                                          //LED Right Direction Status
int sw1Pin = 2;                                             //Switch Select Direction
int ThisRead;                                               //Current Read Buffer
int LastRead = HIGH;                                        //Previos Read Buffer
long Debounce;                                              //Debounce Time Buffer
bool DirStatus=LOW;                                         //Direction Status Buffer(Toggle)

void setup()
{
  pinMode(IN1, OUTPUT);                                     //IN1 = Output
  pinMode(EN, OUTPUT);                                      //EN  = Output
  pinMode(IN2, OUTPUT);                                     //IN2 = Output
  pinMode(ledLeft,OUTPUT);                                  //LED Left Direction = Output
  pinMode(ledRight,OUTPUT);                                 //LED Right Direction = Output
  pinMode(sw1Pin, INPUT);                                   //Switch = Input
  
  digitalWrite(IN1, HIGH);                                  //Direction = Break
  digitalWrite(IN2, HIGH);
  digitalWrite(ledLeft, LOW);                               //LED = OFF
  digitalWrite(ledRight, LOW);                              //LED = OFF
}

void loop()                                                 //Main Function
{ 
  do
  { 
    ThisRead = digitalRead(sw1Pin);                         //Read Sw Pin  
    if(ThisRead == LastRead)                                //if Switch Not Change
    {
      Debounce = millis();                                  //Re-Start Debounce Time
      int val = map(analogRead(Volume), 0, 1023, 0, 255);   //Convert 0..1023 to 0..255
      analogWrite(EN,val);                                  //Speed Motor = 0..255 Scale
    }
  }while((millis() - Debounce) < 50);                       //Wait Switch Change > 50mS
        
  if (ThisRead == LOW && LastRead == HIGH)                  //if Sw Press
  {
    DirStatus = !DirStatus;                                 //Toggle Direction
    if(DirStatus)
    {
      digitalWrite(IN1, HIGH);                              //Direction = Break
      digitalWrite(IN2, HIGH);
      delay(500);                                           //Wait Motor Break Ready
      digitalWrite(IN1, LOW);                               //Direction = Left
      digitalWrite(IN2, HIGH);
      digitalWrite(ledLeft, HIGH);                          //LED Left Direction = ON
      digitalWrite(ledRight, LOW);
    }
    else
    {
      digitalWrite(IN1, HIGH);                              //Direction = Break
      digitalWrite(IN2, HIGH);
      delay(500);                                           //Wait Motor Break Ready
      digitalWrite(IN1, HIGH);                              //Direction = Right
      digitalWrite(IN2, LOW);
      digitalWrite(ledLeft, LOW);
      digitalWrite(ledRight, HIGH);                         //LED Right Direction = ON      
    }    
  }  
  LastRead = ThisRead;                                      //Save Last Sw Status    
}
