/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB7.2
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : 1-Wire Temperature DS18S20
 * Function  : Single Device Read 1-Wire Temperature DS18S20 Display RS232
 */
 
#include <OneWire.h>                                  //Used 1-Wire Library
OneWire  ds(2);                                       //1-Wire Interface Pin Digital-2

/*
 * Convert Double to ASCII Decimal Number
 */
char* DoubleToAscii(double num)
{
  char ascii[32];
  int frac;
  
  frac=(int)(num*10)%10;                              //Get 1 Digit of of Deciaml Point
  itoa((int)num,ascii,10);                            //Convert Interger to ASCII (Decimal Base)
  strcat(ascii,".");
  itoa(frac,&ascii[strlen(ascii)],10);                //Convert frac. to ASCII (Decimal Base) 

  return ascii;
}

void setup()                                          //Setup Function
{
  Serial.begin(19200);                                //Initial RS232 Baud = 19200
   Serial.println("Demo 1-Wire : DS18S20(Single Device Read)");
  Serial.println();
}

void loop(void) 
{
  byte data[9];                                       //9 Byte Array Savd Temperature Code    
  int temp;
  
  if(ds.reset())                                      //If Device ACK on Reset
  {    
    ds.write(0xCC);                                   //Skip ROM Command
    ds.write(0x44);                                   //Start Conversion Temp
    
    ds.reset();                                       //Re-Start New Command
    ds.write(0xCC);                                   //Skip ROM Command
    ds.write(0xBE);                                   //Read Scratchpad RAM(Temperature Data)
    
    for ( int i = 0; i<9; i++)                        //9 Byte Read Scratchpad RAM
    {           
      data[i] = ds.read();
    }
    
    if (ds.crc8(data,8) != data[8])                    //Verify 9 Byte CRC(0..8) By CRC-8
    {
      Serial.println("Read Device CRC.....Error");
    }
    else
    { 
      Serial.print("Scratchpad = ");
      for ( int i=0; i<9; i++)                          //9 Byte Print
      {                 
        if (data[i] < 0x10)
        {
          Serial.print("0");                            //Insert 1st Zero
        }  
        Serial.print(data[i], HEX);                     //2nd Digit HEX      
        Serial.print(" ");                              //Seperate Byte By Space
      }
     
      Serial.print(" --> Temperature = ");                 
      temp=(data[1]<<8)+data[0];                        //data[1]MSB+data[0]LSB
      
      if(data[1]!=0x00)                                 //Verify Signed of Temperature : + or - 
      {
        Serial.print("-");                               
        temp = (~temp)+1;                               //2'Complement
      }
      else
      {
        Serial.print("+");    
      }
         
      Serial.print(DoubleToAscii((double)temp*0.5));    //0.5 Degree / Step
      Serial.println(" C");
    }
  }
  else
  {
    Serial.println("Search Device...Not Found");
  }  
  delay(1000);                                          //Wait 1 Second Repeat
}
