/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB8.2
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : ET-MINI DS1307
 *           : -> SDA = PC4(Analog-4) 
 *           : -> SCL = PC5(Analog-5)  
 *           : -> INT = PD2 : INT0(Digital-2)
 * Function  : Read RTC Date,Time & Print to RS232
 *           : Used RTC Interrupt Mode
 */

#include <Wire.h>                               //Used I2C Library

#include "WProgram.h"
void setup();
void loop();
void SetClock();
void GetClock();
byte BCDtoDecimal(byte bcdVal);
void Print2DigitHex(byte hexVal);
void PrintDayName(byte Name);
void PrintMonthName(byte Name);
void RTC_Interrupt();
int ledPin = 13;                                //LED Status
int intPin = 2;                                 //INT0:PD2(Digital-2)
volatile boolean ledStatus = LOW;               //used in Interrupt Service
volatile boolean intStatus = LOW;               //used in Interrupt Service

// DS1307 Clock Registers
#define DS1307      0xD0 >> 1                   //DS1307 ID Code
#define RTC_SECOND  0
#define RTC_MINUTE  1
#define RTC_HOUR    2
#define RTC_WKDAY   3
#define RTC_DATE    4
#define RTC_MONTH   5
#define RTC_YEAR    6
#define RTC_CTRL    7

//DS1307 Clock Buffer
byte second;                                   
byte minute;
byte hour;
byte wkDay;
byte date;
byte month;
byte year;
byte ctrl;

void setup()                                    //Setup Function
{
  Wire.begin();                                 //Initial I2C Bus
  pinMode(ledPin, OUTPUT);                      //ledPin = Output
  pinMode(intPin, INPUT);                       //intPin = Input
  
  //Default Start Date & Time Setup 
  second = 0x55;                                //00..59
  minute = 0x59;                                //00..59
  hour   = 0x23;                                //00..23
  wkDay  = 0x07;                                //Saturday(1..7,Sun,Mon,...,Sat)
  date   = 0x31;                                //1..28,29,30,31
  month  = 0x12;                                //1..12
  year   = 0x08;                                //00..99
  ctrl   = 0x10;                                //Enable SQW Output = 1Hz
  SetClock();                                   //Setup Clock & Time
  
  Serial.begin(19200);                          //Initial RS232 Baud = 19200  
  Serial.println("Demo Test RTC:DS1307 Interrupt Mode");
  Serial.println();
  attachInterrupt(0,RTC_Interrupt, CHANGE);     //Change Edge Interrupt Trigger  
}

void loop()
{
 while(intStatus==0);                           //Wait Interrupt Trigger 
 GetClock();                                    // Read RTC Clock
 
 //Print Time & Data Format = hh:mm:ss day date-month-year
 Print2DigitHex(hour);
 Serial.print(":");
 Print2DigitHex(minute);
 Serial.print(":");
 Print2DigitHex(second);
 Serial.print(" ");
 PrintDayName(BCDtoDecimal(wkDay));            //2-Digit BCD to Decimal 
 Serial.print(" ");
 Print2DigitHex(date);
 Serial.print("-");
 PrintMonthName(BCDtoDecimal(month));          //2-Digit BCD to Decimal        
 Serial.print("-20");                          //Insert Year Name = 20xx
 Print2DigitHex(year);
 Serial.println();
 
 intStatus = 0;                                //Reset Interrupt Status
}

void SetClock()
{
 Wire.beginTransmission(DS1307);
 Wire.send(RTC_SECOND);
 Wire.send(second);
 Wire.send(minute);
 Wire.send(hour);
 Wire.send(wkDay);
 Wire.send(date);
 Wire.send(month);
 Wire.send(year);
 Wire.send(ctrl);
 Wire.endTransmission();
}

void GetClock()
{
 Wire.beginTransmission(DS1307);  
 Wire.send(RTC_SECOND);
 Wire.endTransmission();            
 Wire.requestFrom(DS1307, 8);      //8 Byte Read
 second = Wire.receive();
 minute = Wire.receive();
 hour   = Wire.receive();
 wkDay  = Wire.receive();
 date   = Wire.receive();
 month  = Wire.receive();
 year   = Wire.receive();
 ctrl   = Wire.receive();
}

byte BCDtoDecimal(byte bcdVal)
{
 return bcdVal / 16 * 10 + bcdVal % 16;
}

void Print2DigitHex(byte hexVal)
{
  if (hexVal < 0x10)
  {
    Serial.print("0");
  }  
  Serial.print(hexVal, HEX);
}

void PrintDayName(byte Name)
{
  switch (Name) 
  {
    case 1: Serial.print("Sunday");
            break;
    case 2: Serial.print("Monday");
            break;
    case 3: Serial.print("Tuesday");
            break;
    case 4: Serial.print("Wednusday");
            break;
    case 5: Serial.print("Thursday");
            break;
    case 6: Serial.print("Friday");
            break;
   case 7:  Serial.print("Saturday");
            break;
   default: Serial.print("???");
 }
}

void PrintMonthName(byte Name)
{
  switch (Name) 
  {
    case 1:  Serial.print("January");
             break;
    case 2:  Serial.print("February");
             break;
    case 3:  Serial.print("March");
             break;
    case 4:  Serial.print("April");
             break;
    case 5:  Serial.print("May");
             break;
    case 6:  Serial.print("June");
             break;
    case 7:  Serial.print("July");
             break;
    case 8:  Serial.print("August");
             break;
    case 9:  Serial.print("September");
             break;
    case 10: Serial.print("October");
             break;
    case 11: Serial.print("November");
             break;
    case 12: Serial.print("December");
             break;
    default: Serial.print("???");
 }
} 

void RTC_Interrupt()                              //Interrupt Service Function
{
  if(digitalRead(intPin)==LOW)                    //If Falling Edge Interrupt
  {
    intStatus = HIGH;                             //Active Interrupt Status
    digitalWrite(ledPin,HIGH);                    //LED Interrupt ON
  }
  else                                            //If Rising Edge Interrupt
  {
    digitalWrite(ledPin,LOW);                     //LED Interrupt OFF
  }
}

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

