/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB8.4
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : ET-MINI PCF8583
 *           : -> SDA = PC4(Analog-4) 
 *           : -> SCL = PC5(Analog-5)  
 * Function  : Read RTC Date,Time & Print to RS232
 *           : Demo Interrupt Read RTC
 */

#include <Wire.h>                                       //Used I2C Library

int ledPin = 13;                                        //LED Status
int intPin = 2;                                         //INT0:PD2(Digital-2)
volatile boolean ledStatus = LOW;                       //used in Interrupt Service
volatile boolean intStatus = LOW;                       //used in Interrupt Service

// PCF8583 Clock Registers
#define PCF8583        0xA2 >> 1                        //PCF8583 ID Code
#define RTC_CTRL       0
#define RTC_SEC_100    1                               
#define RTC_SEC        2
#define RTC_MIN        3
#define RTC_HOUR       4
#define RTC_YEAR_DATE  5
#define RTC_WEEK_MONTH 6
#define RTC_TIMER      7
#define ALM_CTRL       8
#define ALM_SEC        10
#define ALM_MIN        11
#define ALM_HOUR       12
#define ALM_DATE       13
#define ALM_MONTH      14
#define ALM_TIMER      15


//PCF8583 Clock Buffer
byte second;                                   
byte minute;
byte hour;
byte date;
byte month;

void setup()                                            //Setup Function
{
  Serial.begin(19200);                                  //Initial RS232 Baud = 19200
  Wire.begin();                                         // Initial I2C Bus
  pinMode(ledPin, OUTPUT);                              //ledPin = Output
  pinMode(intPin, INPUT);                               //intPin = Input
  
  //Default Start Date & Time Setup 
  second = 0x55;                                        //00..59
  minute = 0x59;                                        //00..59
  hour   = 0x23;                                        //00..23  
  date   = 0x31;                                        //1..28,29,30,31
  month  = 0x12;                                        //1..12
  SetClock();                                           //Setup Clock & Time  
  
  WriteDevice(RTC_CTRL,0x08);                           //Config RTC Mode,Mask date&month(00001000)
  WriteDevice(ALM_CTRL,0x80);                           //Enable Interrupt 1-Second (10000000)
  
  Serial.println("Demo RTC:PCF8583 Interrupt Mode");
  Serial.println();
  attachInterrupt(0,RTC_Interrupt, CHANGE);             //Change Edge Interrupt Trigger  
}

void loop()
{
 while(intStatus==0);                                   //Wait Interrupt Trigger 
 GetClock();                                            // Read RTC Clock
 
 //Print Time & Data Format = hh:mm:ss date-month
 Print2DigitHex(hour);
 Serial.print(":");
 Print2DigitHex(minute);
 Serial.print(":");
 Print2DigitHex(second);
 Serial.print(" ");
 Print2DigitHex(date);
 Serial.print("-");
 PrintMonthName(BCDtoDecimal(month));                  //2-Digit BCD to Decimal        
 Serial.println();
 
 intStatus = 0;                                        //Reset Interrupt Status
}

void WriteDevice(byte addr,byte data)
{
 Wire.beginTransmission(PCF8583);
 Wire.send(addr);                                      //Start Write Address 
 Wire.send(data);                                      //Data 
 Wire.endTransmission();
}

void SetClock()
{
 Wire.beginTransmission(PCF8583);
 Wire.send(RTC_SEC);                                 //Start Write From RTC_SEC 
 Wire.send(second);                                  //Second
 Wire.send(minute);                                  //Minute
 Wire.send(hour);                                    //Hour
 Wire.send(date);                                    //Date
 Wire.send(month);                                   //Month
 Wire.endTransmission();
}

void GetClock()
{
 Wire.beginTransmission(PCF8583);  
 Wire.send(RTC_SEC);                                //Start Read From RTC_SEC
 Wire.endTransmission();            
 Wire.requestFrom(PCF8583, 5);                      //5 Byte Read
 second = Wire.receive();                           //Second 10:1
 minute = Wire.receive();                           //Minute 10:1
 hour   = Wire.receive();                           //Hour   10:1
 date   = Wire.receive();                           //Date   00+dddddd
 month  = Wire.receive();                           //Month  000+mmmmm 
}

byte BCDtoDecimal(byte bcdVal)
{
 return bcdVal / 16 * 10 + bcdVal % 16;
}

void Print2DigitHex(byte hexVal)
{
  if (hexVal < 0x10)
  {
    Serial.print("0");
  }  
  Serial.print(hexVal, HEX);
}

void PrintMonthName(byte Name)
{
  switch (Name) 
  {
    case 1:  Serial.print("January");
             break;
    case 2:  Serial.print("February");
             break;
    case 3:  Serial.print("March");
             break;
    case 4:  Serial.print("April");
             break;
    case 5:  Serial.print("May");
             break;
    case 6:  Serial.print("June");
             break;
    case 7:  Serial.print("July");
             break;
    case 8:  Serial.print("August");
             break;
    case 9:  Serial.print("September");
             break;
    case 10: Serial.print("October");
             break;
    case 11: Serial.print("November");
             break;
    case 12: Serial.print("December");
             break;
    default: Serial.print("???");
 }
} 

void RTC_Interrupt()                                  //Interrupt Service Function
{
  if(digitalRead(intPin)==LOW)                        //If Falling Edge Interrupt
  {
    intStatus = HIGH;                                 //Active Interrupt Status
    digitalWrite(ledPin,HIGH);                        //LED Interrupt ON
  }
  else                                                //If Rising Edge Interrupt
  {
    digitalWrite(ledPin,LOW);                         //LED Interrupt OFF
  }
}
