/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB8.5
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : ET-MINI PCF8583
 *           : -> SDA = PC4(Analog-4) 
 *           : -> SCL = PC5(Analog-5)  
 * Function  : Read RTC Date,Time & Print to RS232
 *           : Demo RTC Alarm Interrupt
 */

#include <Wire.h>                                       //Used I2C Library

int ledPin = 13;                                        //LED Status
int intPin = 2;                                         //INT0:PD2(Digital-2)
volatile boolean ledStatus = LOW;                       //used in Interrupt Service
volatile boolean intStatus = LOW;                       //used in Interrupt Service

// PCF8583 Clock Registers
#define PCF8583        0xA2 >> 1                        //PCF8583 ID Code
#define RTC_CTRL       0
#define RTC_SEC_100    1                               
#define RTC_SEC        2
#define RTC_MIN        3
#define RTC_HOUR       4
#define RTC_YEAR_DATE  5
#define RTC_WEEK_MONTH 6
#define RTC_TIMER      7
#define ALM_CTRL       8
#define ALM_SEC_100    9
#define ALM_SEC        10
#define ALM_MIN        11
#define ALM_HOUR       12
#define ALM_DATE       13
#define ALM_MONTH      14
#define ALM_TIMER      15


//PCF8583 Clock Buffer
byte clock_second;                                   
byte clock_minute;
byte clock_hour;
byte alarm_second;                                   
byte alarm_minute;
byte alarm_hour;

void setup()                                            //Setup Function
{
  Serial.begin(19200);                                  //Initial RS232 Baud = 19200
  Wire.begin();                                         // Initial I2C Bus
  pinMode(ledPin, OUTPUT);                              //ledPin = Output
  pinMode(intPin, INPUT);                               //intPin = Input
  
  //Default Start Date & Time Setup 
  clock_second = 0x00;                                  //00..59
  clock_minute = 0x00;                                  //00..59
  clock_hour   = 0x00;                                  //00..23    
  SetClock();                                           //Setup Clock & Time  
  
  alarm_second = 0x10;                                  //00..59
  alarm_minute = 0x00;                                  //00..59
  alarm_hour   = 0x00;                                  //00..23    
  SetAlarm();                                           //Setup Alarm   
  
  WriteDevice(RTC_CTRL,0x0C);                           //Config RTC Mode,Mask date&month,Enable Alarm(00001100)
  WriteDevice(ALM_CTRL,0x90);                           //Enable Interrupt Alarm Daily (10010000)
  
  Serial.println("Demo RTC:PCF8583 Alarm Interrupt");
  Serial.println();
  attachInterrupt(0,RTC_Interrupt, CHANGE);             //Change Edge Interrupt Trigger  
}

void loop()
{
 while(intStatus==0)                                    //Wait Interrupt Trigger 
 {
   GetClock();                                            // Read RTC Clock
 
   //Print Time & Data Format = hh:mm:ss date-month
   Print2DigitHex(clock_hour);
   Serial.print(":");
   Print2DigitHex(clock_minute);
   Serial.print(":");
   Print2DigitHex(clock_second);
   Serial.println();
   
   delay(1000);
 }
 
 intStatus = 0;                                        //Reset Interrupt Status
 Serial.println();
 Serial.println("*****Alarm Active*****");
 Serial.println();
 while(1);                                            //Stop Program
}

void WriteDevice(byte addr,byte data)
{
 Wire.beginTransmission(PCF8583);
 Wire.send(addr);                                      //Start Write Address 
 Wire.send(data);                                      //Data 
 Wire.endTransmission();
}

void SetClock()
{
 Wire.beginTransmission(PCF8583);
 Wire.send(RTC_SEC);                                 //Start Write From RTC_SEC 
 Wire.send(clock_second);                            //Second
 Wire.send(clock_minute);                            //Minute
 Wire.send(clock_hour);                              //Hour
 Wire.endTransmission();
}

void SetAlarm()
{
 Wire.beginTransmission(PCF8583);
 Wire.send(ALM_SEC);                                 //Start Write From ALM_SEC
 Wire.send(alarm_second);                            //Second
 Wire.send(alarm_minute);                            //Minute
 Wire.send(alarm_hour);                              //Hour
 Wire.endTransmission();
}

void GetClock()
{
 Wire.beginTransmission(PCF8583);  
 Wire.send(RTC_SEC);                                //Start Read From RTC_SEC
 Wire.endTransmission();            
 Wire.requestFrom(PCF8583, 3);                      //3 Byte Read
 clock_second = Wire.receive();                     //Second 10:1
 clock_minute = Wire.receive();                     //Minute 10:1
 clock_hour   = Wire.receive();                     //Hour   10:1 
}

byte BCDtoDecimal(byte bcdVal)
{
 return bcdVal / 16 * 10 + bcdVal % 16;
}

void Print2DigitHex(byte hexVal)
{
  if (hexVal < 0x10)
  {
    Serial.print("0");
  }  
  Serial.print(hexVal, HEX);
}

void RTC_Interrupt()                                  //Interrupt Service Function
{
  if(digitalRead(intPin)==LOW)                        //If Falling Edge Interrupt
  {
    intStatus = HIGH;                                 //Active Interrupt Status
    digitalWrite(ledPin,HIGH);                        //LED Interrupt ON
  }
  else                                                //If Rising Edge Interrupt
  {
    digitalWrite(ledPin,LOW);                         //LED Interrupt OFF
  }
}
