/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB8.6
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : ET-MINI 24XX(24LC32)
 *           : -> SDA = PC4(Analog-4) 
 *           : -> SCL = PC5(Analog-5)  
 * Function  : Demo Write/Read I2C EEPROM & Print to RS232
 */

#include <Wire.h>                                                 //Used I2C Library

// 24LC32 EEPROM ID Code
#define EEP_DEVICE0 0xA8 >> 1                                     //EEPROM 24LC32 ID Code(1010100X)
#define EEP_DEVICE1 0xAA >> 1                                     //EEPROM 24LC32 ID Code(1010101X)
#define EEP_DEVICE2 0xAC >> 1                                     //EEPROM 24LC32 ID Code(1010110X)
#define EEP_DEVICE3 0xAE >> 1                                     //EEPROM 24LC32 ID Code(1010111X)

//Array String for Test Write Page EEPROM(MAX 30Byte)
byte msg[4][20] = {"This is Data Test-0", 
                   "This is Data Test-1", 
                   "This is Data Test-2", 
                   "This is Data Test-3"};                  
byte buf[20];                                   

/*
 * Write 1 Byte to I2C EEPROM
 */
void i2c_eeprom_write_byte(int dev_addr, unsigned int eep_addr, byte data) 
{
  int rdata = data;
  Wire.beginTransmission(dev_addr);                              //Device ID
  Wire.send((int)(eep_addr >> 8));                               //MSB Address
  Wire.send((int)(eep_addr & 0xFF));                             //LSB Address
  Wire.send(rdata);                                              //Data
  Wire.endTransmission();                                        //Stop Condition
}

/*
 * Read Page to I2C EEPROM(MAX 30 Byte)
 */
void i2c_eeprom_write_page(int dev_addr, unsigned int eep_addr_page, byte* data, byte length) 
{
  Wire.beginTransmission(dev_addr);                              //Device ID
  Wire.send((int)(eep_addr_page >> 8));                          //MSB Address
  Wire.send((int)(eep_addr_page & 0xFF));                        //LSB Address 
  
  for (int c=0; c<length; c++)                                   //Page Size Write
  {
    Wire.send(data[c]);
  }  
  Wire.endTransmission();                                        //Stop Condition
}

/*
 * Read 1 Byte From I2C EEPROM
 */
byte i2c_eeprom_read_byte(int dev_addr, unsigned int eep_addr) 
{
  byte rdata = 0xFF;                                            //Dummy Buffer
  Wire.beginTransmission(dev_addr);                             //Device ID
  Wire.send((int)(eep_addr >> 8));                              //MSB Address
  Wire.send((int)(eep_addr & 0xFF));                            //LSB Address
  Wire.endTransmission();                                       //Stop Condition
  
  Wire.requestFrom(dev_addr,1);                                 //Device Address Read
  if (Wire.available())                                         //Wait Data Ready
  {
    rdata = Wire.receive();                                     //Read Byte Data
  }
  return rdata;                               
}

/*
 * Read Page From I2C EEPROM(MAX 30 Byte)
 */
void i2c_eeprom_read_buffer(int dev_addr, unsigned int eep_addr, byte *buffer, int length) 
{
  Wire.beginTransmission(dev_addr);                              //Device ID
  Wire.send((int)(eep_addr >> 8));                               //MSB Address
  Wire.send((int)(eep_addr & 0xFF));                             //LSB Address
  Wire.endTransmission();                                        //Stop Condition
  
  Wire.requestFrom(dev_addr,length);                             //Page Size Read 
  for (int c=0; c<length; c++ )
  {
    if (Wire.available())                                        //Wait Data Ready
    { 
      buffer[c] = Wire.receive();                                //Read Byte Data to Buffer
    }
  }
}

void setup()                                                      //Setup Function
{
  Wire.begin();                                                   //Initial I2C Bus
  Serial.begin(19200);                                            //Initial RS232 Baud = 19200    
  Serial.println();
  Serial.println("Demo Write/Read I2C EEPROM 24LC32");
  Serial.println();
}

void loop()
{ 
  Serial.println("=========================================================================");
  Serial.println("Read I2C EEPROM Address[0...19]");
  
  Serial.print("HEX  Data > ");  
  for (int i = 0; i<20; i++)               
  { 
    Serial.print(i2c_eeprom_read_byte(EEP_DEVICE0,i),HEX);        //Test Read Byte Mode
    Serial.print(" ");
  }
  Serial.println();
  
  Serial.print("Char Data > ");  
  i2c_eeprom_read_buffer(EEP_DEVICE0,0,buf,20);                    //Test Read Page Device[0]
  for (int i = 0; i<20; i++)               
  {                     
    Serial.print(buf[i]);                                          //Display Character
    Serial.print("  ");
  }
  Serial.println();
  Serial.println("=========================================================================");
  Serial.println("Select Test Write I2C EEPROM");
  Serial.print("0. Test Write = ");
  Serial.println((char*)msg[0]);
  Serial.print("1. Test Write = ");
  Serial.println((char*)msg[1]);
  Serial.print("2. Test Write = ");
  Serial.println((char*)msg[2]);
  Serial.print("3. Test Write = ");
  Serial.println((char*)msg[3]);
  Serial.println("=========================================================================");
  Serial.print("Select....");
  
  //Wait & Select Choice Test Write EEPROM
  while(Serial.available()==0);                                 //Wait Choice [0..3]  
  int var = Serial.read();   
  Serial.println(var,BYTE);     
  Serial.println();
  switch(var)
  {
    case '0' :  i2c_eeprom_write_page(EEP_DEVICE0,0,msg[0],sizeof(msg[0]));
                break;
    
    case '1' :  i2c_eeprom_write_page(EEP_DEVICE0,0,msg[1],sizeof(msg[1]));
                break;
    
    case '2' :  i2c_eeprom_write_page(EEP_DEVICE0,0,msg[2],sizeof(msg[2]));
                break;
    
    case '3' :  i2c_eeprom_write_page(EEP_DEVICE0,0,msg[3],sizeof(msg[3]));
                break;
  }
}


