/*
 * Examples  : Arduino Experiment Examples By....ETT CO.,LTD
 * Program   : Hardware LAB9.3
 * Hardware  : ET-BASE AVR EASY MEGA88/168(Arduino)
 *           : ET-MINI MCP4922(DAC 12BIT 2 Channel)
 *           : -> CS#   = Digital-8
 *           : -> SCK   = Digital-9
 *           : -> SDI   = Digital-10
 *           : -> LDAC# = Digital-11
 *           : -> SHDN# = Digital-12
 * Function  : Demo SPI DAC MCP4922(DAC 12BIT 2 Channel)
 *           : Control DAC MCP4922 Output = Sine Wave 
 */

int CS   = 8;                                        
int SCK  = 9;                                        
int SDI  = 10;                                        
int LDAC = 11;
int SHDN = 12;

// Array[0..64] = 64 Step = Degree[0..360]/64 = 5.625/Step
// Step Degree[0..63] = 0,5.625,11.25,16.875,...,354.375,360
// Step Output[0..63] = (sin(x)*2048)+2048
// Example = (sin(5.625)*2048)+2048
//         = (0.098*2048)+2048
//         = 2248.739 
//         = 2048 
//         = 0x08C8
int table_sine[64] = 	                // Sine Function Table
	      {
 	        0x07FF, 0x08C8, 0x098E, 0x0A51, 	
		0x0B0F, 0x0BC4, 0x0C71, 0x0D12,
 	        0x0DA7, 0x0E2E, 0x0EA5, 0x0F0D, 
		0x0F63, 0x0FA6, 0x0FD7, 0x0FF5,
 		0x0FFF, 0x0FF5, 0x0FD7, 0x0FA6, 
		0x0F63, 0x0F0D, 0x0EA5, 0x0E2E,
 	        0x0DA7, 0x0D12, 0x0C71, 0x0BC4, 
		0x0B0F, 0x0A51, 0x098E, 0x08C8,
 	        0x07FF, 0x0736, 0x0670, 0x05AD, 
		0x04EF, 0x043A, 0x038D, 0x02EC,
 	        0x0257, 0x01D0, 0x0159, 0x00F1, 
		0x009B, 0x0058, 0x0027, 0x0009,
 	        0x0000, 0x0009, 0x0027, 0x0058, 
		0x009B, 0x00F1, 0x0159, 0x01D0,
 	        0x0257, 0x02EC, 0x038D, 0x043A, 
		0x04EF, 0x05AD, 0x0670, 0x0736  
	      };

void Write_MCP4922(int DAC_Channel,int DAC_Data)	//Write DAC Output
{                                     
  digitalWrite(CS, LOW); 				//Enable MCP4922 

  switch (DAC_Channel)					//Select DAC Channel(0=A,1=B)						
  {
    case 0x00: DAC_Data |= 0x3000;			//Write DAC-A
               break;
    case 0x01: DAC_Data |= 0xB000;			//Write DAC-B
               break;
  }  
  shiftOut(SDI, SCK, MSBFIRST,(DAC_Data >> 8)& 0xFF);   //MSB Data(High Byte)
  shiftOut(SDI, SCK, MSBFIRST,DAC_Data & 0xFF); 	//LSB Data(Low Byte)

  digitalWrite(CS, HIGH); 				//Disable MCP4922
    
  digitalWrite(LDAC, LOW); 				//Enable Pulse Latch
  digitalWrite(LDAC, HIGH); 
}   

void setup() 
{
  pinMode(CS, OUTPUT);                              
  pinMode(SCK, OUTPUT);                             
  pinMode(SDI, OUTPUT); 
  pinMode(LDAC, OUTPUT);                             
  pinMode(SHDN, OUTPUT);  
  
  digitalWrite(CS, HIGH);                               //Standby CS Signal
  digitalWrite(LDAC, HIGH);                             //Standby LDAC Signal
  digitalWrite(SHDN, HIGH);                             //Standby SHDN Signal
}

void loop() 
{
  for(int i=0; i<64; i++)
  {     
    Write_MCP4922(0,table_sine[i]); 	                //Update DAC0 Sine Output
    Write_MCP4922(1,(int)(sin(((i*(360.0/64.0))*3.14159/180.0))*2047.0)+2047.0); 
  }   
} 
