/*
 * Examples  : Arduino USB Host Examples By....ETT CO.,LTD
 * Program   : Android_Interface(ADB Microbridge Library)
 * Hardware  : ET-MEGA2560-ADK(Arduino)
 * Function  : Demo USB Host Interface Android Device V1.5++
 */

// Use Output on LED(D13)
// Use Analog Input: AN0 Read Analog Scale 0..1023

#include <SPI.h>
#include <Adb.h>                         // Please Used ADB Library Version Modify By ETT for ET-MEGA2560-ADK

Connection * connection;                 // Adb connection.
int ledPin = 13;                         // ledPin = Digital Pin-13
int  Volume = 0;                         // Volume = Analog Pin-0
long lastTime;                           // Elapsed time for ADC sampling. The rate at which ADC value is sent to Android device.
uint8_t LEDState=0;                      // State of LED. Initially OFF. 

// Event handler for the shell connection. 
// This event handler is called whenever data is sent from Android Device to Seeeduino ADK.
// Any data / command to be sent to I/O of ADK has to be handled here.
// For eg: 1.Controlling an ouput port 2.Interacting with a device connected
// to ADK via IIC or Serial Port.
void adbEventHandler(Connection * connection, adb_eventType event, uint16_t length, uint8_t * data)
{  
  // In this example Data packets contain one byte and it decides the state of a LED connected to D13
  // The size of data is predetermined for this application. Android device also uses the same size.

  if (event == ADB_CONNECTION_RECEIVE)
  {
     if(LEDState != data[0])
     {
         digitalWrite(ledPin, data[0]);  // Change the state of LED 
         Serial.println(data[0],DEC); 
       	 LEDState = data[0];             // Store the State of LED
     }	
  }
}

void setup()
{
  
  Serial.begin(115200);                  // Serial port debug purpose
  lastTime = millis();                   // Note start time
  pinMode(ledPin, OUTPUT);               // ledPin = Output  
  ADB::init();                           // Initialise the ADB subsystem.  

  // Open an ADB stream to the phone's shell. Auto-reconnect. Use any unused port number eg:4568
  connection = ADB::addConnection("tcp:4568", true, adbEventHandler);  
}

void loop()
{
  
  if ((millis() - lastTime) > 20)          // 20mS Repeat Sampling ADC
  {    
    uint16_t data = analogRead(Volume);    // Read ADC value    
    connection->write(2,(uint8_t*)&data);  // Send the ADC value to Android device as two bytes of data.
    lastTime = millis();
  }
  
  ADB::poll();                             // Poll the ADB subsystem.
}

