package com.arduinoADK.Mega2560ADK;

import java.io.IOException;

import org.microbridge.server.AbstractServerListener;
import org.microbridge.server.Server;

//Receive From ET-MEGA2560-ADK
import com.arduinoADK.Mega2560ADK.R;

import android.view.View;
//import android.view.Window;
import android.view.View.OnClickListener;
import android.app.Activity;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Button;

public class Mega2560ADKV233Demo extends Activity implements OnClickListener 
{
  //Variable to store input from Arduino A0
  private int adcA0Value=0;
  	
  //Below are UI variables
  Button btnLEDControl;
  
  SeekBar sb;
  TextView tv;
  
  boolean LEDState = false ; //initially OFF
  
  //microbridge server
  Server server = null;
			
  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState) 
  {
    super.onCreate(savedInstanceState);
     
    /* Create UI */
    // Set full screen view, no title
    //getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
    //requestWindowFeature(Window.FEATURE_NO_TITLE);
    setContentView(R.layout.main);
    
    btnLEDControl = (Button) findViewById(R.id.myButtonLED);
    btnLEDControl.setOnClickListener(this); 
    
    /* Create TCP server Using microbridge*/
    /*  What is microbridge? - Text from - http://code.google.com/p/microbridge/ 
    	MicroBridge is an Android Debug Bridge (ADB) implementation for microcontrollers. 
    	MicroBridge allows stock, unrooted Android devices to talk directly to USB host 
    	enabled MCUs, thereby enabling phones to actuate servos, drive DC motors, talk to I2C 
    	and SPI devices, read ADCs, and so forth. MicroBridge works on Android 1.5 and upwards.
    */
     		     		
    try
    {
      //server = new Server(4569); //No other app should be running on this port
      server = new Server(4568); //Use the same port number used in ADK Main 
      server.start();
    } catch (IOException e)
    {
      Log.e("ET-MEGA2560-ADK : ", "Unable to start TCP server", e);
      System.exit(-1);
    }
    	
   
    server.addListener(new AbstractServerListener() 
    {
      @Override
      public void onReceive(org.microbridge.server.Client client, byte[] data)
      {
    	if (data.length<2) return;    				
    	adcA0Value = (data[0] & 0xff) | ((data[1] & 0xff) << 8);
     				
    	//Update to UI via runOnUIThread
    	runOnUiThread(new Runnable() 
    	{
    	  @Override
    	  public void run() 
    	  {
    		new UpdateData().execute(adcA0Value);     
    	  }
        }); 
      }     
	});//end addListener  		
  }//End of OnCreate
    
    
  // UpdateData Asynchronously sends the value received from ADK Main Board. 
  // This is triggered by onReceive()
  class UpdateData extends AsyncTask<Integer, Integer, String> 
  {
    //Called to initiate the background activity
 	@Override
 	protected String doInBackground(Integer... sensorValue) 
 	{ 	  
 	  SeekBar sb = (SeekBar) findViewById(R.id.mySeekBarADCValue);	    	
 	  sb.setProgress(sensorValue[0]);     	
 	  return (String.valueOf(sensorValue[0]));  //This goes to result 			
 	}
 		
    //Called once the background activity has completed
 	@Override
 	protected void onPostExecute(String result) 
 	{ 	  
 	  TextView tv = (TextView) findViewById(R.id.txtViewADCValue);
 	  tv.setText(String.valueOf(result)); 
 	}
  }
      
  //Called when the LED button is clicked
  @Override
  public void onClick(View v) 
  {  	
  	byte data; 

    // Toggle the ON/OFF state of LED
 	if(LEDState == true)
 	{
 	  LEDState = false;
 	  data = 0;								//OFF LED Status for ET-MEGA2560-ADK 	 
 	  btnLEDControl.setText("LED Off");
 	}
 	else
 	{
 	  LEDState = true;
 	  data = 1;								//ON  LED Status for ET-MEGA2560-ADK 	  
 	  btnLEDControl.setText("LED  On");
 	}
   
  	try
  	{
  	  //Send the state of LED to ADK Main Board as a byte  	  
  	  server.send(new byte[] {(byte) data});
  	} catch (IOException e)
  	{
  	  Log.e("ET-MEGA2560-ADK : ", "problem sending TCP message", e);
  	}   
  }//End of OnClick    
}
